"forward" <-
function(x, p, a, du, pi, step, x.end, x.end.idx, penalty="BIC", b.limit=0.001,
 cut=0.5){

    check.input(x, p, a, du, pi)
    
    ## number of observiations
    n = length(x)
    
    ## maximum duration
    D = nrow(du)
    
    ## number of states
    m = nrow(a)

    ## make sure p start from a value < step
    p = p - floor(min(p)/step)*step
    
    ## how many steps altogether
    Z = ceiling(max(p)/step)
    
    ## step indicator, every 'step' bp is one step
    stp = ceiling(p/step)
    
    ## check that for the input data, every step have at least 3 observations
    len = tapply(p, stp, length)
    if(any(len<3)){
        stop("there is at least one step with less than 3 observations\n")
    }
    
    ## indicator of end of every step
    ## minus 1 to mach the index in c
    dif = diff(stp)
    stp.end = c(which(dif>0), length(stp)) - 1

    ## end of each step
    xR.end = x.end[[1]]
    xR.idx = x.end.idx[[1]]
    for(i in 2:m){
        xR.end = rbind(xR.end, x.end[[i]])
        xR.idx = rbind(xR.idx, x.end.idx[[i]])
    }

    fR = matrix(0, nrow=m*Z, ncol=Z)
    
    if(penalty=="BIC"){
        penal = 2
    }else if(penalty=="AIC"){
        penal = 1
    }else{
        penal = 0
    }
    
    dims = c(n, m, D, Z, step)

    ## indicator of duration 
    di = (du>0)

    SW = .C("forward", as.double(x), as.double(p), as.integer(stp.end), 
            as.integer(dims), f=as.double(t(fR)), as.double(t(a)), 
            as.double(t(du)),  as.integer(t(di)), as.double(t(xR.end)), 
            as.integer(t(xR.idx)), as.double(pi), as.integer(penal), 
            as.double(b.limit), as.double(cut), PACKAGE="ss.hmm");

    fR = matrix(SW$f, byrow=TRUE, nrow=m*Z, ncol=Z)
    
    f = list()
    for(i in 1:m){
        f[[i]] = fR[((i-1)*Z+1):(i*Z),]
    }
    
    f
}

